/* Programa para controlar la secuencia de iluminación de leds con un pulsador
 * ¿Qué hemos visto de nuevo en este programa?
 * - Uso de Arrays para optimizar el funcionamiento del programa
 * 
 * La información de este programa la encuentras en:
 * https://robertogomez.info/aprende-a-usar-pulsadores-de-la-forma-correcta-en-arduino
 * 
 * Puedes muchos más recursos útiles de arduino en: 
 * https://robertogomez.info/blog
 */

//Definimos el pin del pulsador
const int inputPulsador = 7;

//Definimos cuales serán los pines de los leds
const int ledSecuencia1 = 2;
const int ledSecuencia2 = 4;
const int ledSecuencia3 = 8;
const int ledSecuencia4 = 12;
const int ledArduinoPin13 = 13;

//Definimos un Array con los leds a controlar en la secuencia
const int arraySecuenciaLeds[] = {ledSecuencia1, ledSecuencia2, ledSecuencia3, ledSecuencia4, ledArduinoPin13};

const int tiempoTemporizado = 200; // tiempo prefijado en el que alternamos la secuencia de iluminación de los leds 
                             //se expresa en milisegundos ( 1 segundo es igual a 1000 milisegundos)


void setup() {
  
//definimos los pines que vamos a utilizar, y el modo cómo vamos a usarlos, de entrada o de salida
  pinMode(ledSecuencia1, OUTPUT);
  pinMode(ledSecuencia2, OUTPUT);
  pinMode(ledSecuencia3, OUTPUT);
  pinMode(ledSecuencia4, OUTPUT);
  pinMode(ledArduinoPin13, OUTPUT);
  pinMode(inputPulsador, INPUT);
 
  //Para activar la resistencia interna PULL UP en el pin seleccionado de Arduino
  //pinMode(inputPulsador, INPUT_PULLUP); //Verificar la conexión del pulsador según hemos explicado en la entrada del blog
}

void loop() {

  bool valorPulsador = digitalRead(inputPulsador);  
 
  while (valorPulsador == HIGH) {
    secuenciaLEDS();
    valorPulsador = digitalRead(inputPulsador); //Leemos valor de pulsador despues de la secuencia para que el código 
                                                //no se repita una segunda vez si hemos dejado de accionar el pulsador
  }
 
}

void secuenciaLEDS () {
  
  int contador = 0; //variable para recorrer las salidas de los leds a iluminar
  int variableParpadeo = 0;
  int sizeArraySecuenciaLeds = sizeof(arraySecuenciaLeds)/sizeof(int); //Calculamos el tamaño del Array


  //Primer bucle para iluminar los leds en sentido descendente, uno a uno
  for (int x = 0; x < sizeArraySecuenciaLeds - 1; x++){ //Tamaño de Array - 1 para no iluminar el led del pin 13
    digitalWrite(arraySecuenciaLeds[contador - 1], LOW);
    digitalWrite(arraySecuenciaLeds[contador], HIGH);
    contador++;
    delay(tiempoTemporizado);
  }

  //Segundo bucle para iluminar los leds en sentido inverso, en orden ascendente, uno a uno
  for (int x = 0; x < sizeArraySecuenciaLeds - 1; x++){ //Tamaño de Array - 1 para no iluminar el led del pin 13
    digitalWrite(arraySecuenciaLeds[contador], LOW);
    contador--;
    digitalWrite(arraySecuenciaLeds[contador], HIGH);
    delay(tiempoTemporizado);
  }

  //Tercer bucle para el parpadeo de los leds, alternando todos los leds encendidos y todos los leds apagados
  for (int x = 0; x < 8; x++){ //El bucle se repite 8 veces, que son 4 parpadeos de leds
    if (variableParpadeo == 0) {
      for (int y = 0; y < sizeArraySecuenciaLeds; y++){ //Usamos un bucle para poner todos los leds a nivel alto
      digitalWrite(arraySecuenciaLeds[y], HIGH);
      }
      variableParpadeo++;
    } else {
      for (int y = 0; y < sizeArraySecuenciaLeds; y++){ //Usamos un bucle para poner todos los leds a nivel bajo
      digitalWrite(arraySecuenciaLeds[y], LOW);
      }
      variableParpadeo--;
    }
  delay(tiempoTemporizado);
  }
  
}
